@@ -1,6 +1,7 @@
 use crate::data::scalar::Scalar;
 use crate::expr::flexible::FlexibleExpr;
-use crate::expr::flexible::{Boolish, Iterable, Numeric, Stringy, StringyWithList};
+use crate::expr::flexible::{Boolish, Iterable, Numeric, Ordered, Stringy, StringyWithList};
+use pyo3::exceptions::PyTypeError;
 use pyo3::prelude::*;
 
 #[derive(Debug, Clone, Copy, PartialEq)]
@@ -269,6 +270,16 @@ impl LogicalExpr {
         Ok(format!("{:?}", self))
     }
 
+    /// Logical expressions cannot be used in python boolean expressions,
+    /// since `and` and `or` keywords cannot be overridden via magic methods.
+    ///
+    /// Example: `field("a") and field("b")` evaluates to `field("b")`.
+    fn __bool__(&self) -> PyResult<bool> {
+        Err(PyTypeError::new_err(
+            "Using `and` or `or` keywords with Logical expressions is not supported. Please use `&` or `|` instead.",
+        ))
+    }
+
     fn _expr_eq(&self, other: &LogicalExpr) -> bool {
         self == other
     }
@@ -354,7 +365,7 @@ impl LogicalExpr {
         self.ne(py, other)
     }
 
-    fn lt(&self, py: Python<'_>, other: Numeric) -> PyResult<Self> {
+    fn lt(&self, py: Python<'_>, other: Ordered) -> PyResult<Self> {
         let expr: LogicalExpr = other.into();
 
         Ok(Self::Binary {
@@ -364,15 +375,15 @@ impl LogicalExpr {
         })
     }
 
-    fn __lt__(&self, py: Python<'_>, other: Numeric) -> PyResult<Self> {
+    fn __lt__(&self, py: Python<'_>, other: Ordered) -> PyResult<Self> {
         self.lt(py, other)
     }
 
-    fn __rlt__(&self, py: Python<'_>, other: Numeric) -> PyResult<Self> {
+    fn __rlt__(&self, py: Python<'_>, other: Ordered) -> PyResult<Self> {
         self.gt(py, other)
     }
 
-    fn lte(&self, py: Python<'_>, other: Numeric) -> PyResult<Self> {
+    fn lte(&self, py: Python<'_>, other: Ordered) -> PyResult<Self> {
         let expr: LogicalExpr = other.into();
 
         Ok(Self::Binary {
@@ -382,15 +393,15 @@ impl LogicalExpr {
         })
     }
 
-    fn __le__(&self, py: Python<'_>, other: Numeric) -> PyResult<Self> {
+    fn __le__(&self, py: Python<'_>, other: Ordered) -> PyResult<Self> {
         self.lte(py, other)
     }
 
-    fn __rle__(&self, py: Python<'_>, other: Numeric) -> PyResult<Self> {
+    fn __rle__(&self, py: Python<'_>, other: Ordered) -> PyResult<Self> {
         self.gte(py, other)
     }
 
-    fn gt(&self, py: Python<'_>, other: Numeric) -> PyResult<Self> {
+    fn gt(&self, py: Python<'_>, other: Ordered) -> PyResult<Self> {
         let expr: LogicalExpr = other.into();
 
         Ok(Self::Binary {
@@ -400,15 +411,15 @@ impl LogicalExpr {
         })
     }
 
-    fn __gt__(&self, py: Python<'_>, other: Numeric) -> PyResult<Self> {
+    fn __gt__(&self, py: Python<'_>, other: Ordered) -> PyResult<Self> {
         self.gt(py, other)
     }
 
-    fn __rgt__(&self, py: Python<'_>, other: Numeric) -> PyResult<Self> {
+    fn __rgt__(&self, py: Python<'_>, other: Ordered) -> PyResult<Self> {
         self.lt(py, other)
     }
 
-    fn gte(&self, py: Python<'_>, other: Numeric) -> PyResult<Self> {
+    fn gte(&self, py: Python<'_>, other: Ordered) -> PyResult<Self> {
         let expr: LogicalExpr = other.into();
 
         Ok(Self::Binary {
@@ -418,11 +429,11 @@ impl LogicalExpr {
         })
     }
 
-    fn __ge__(&self, py: Python<'_>, other: Numeric) -> PyResult<Self> {
+    fn __ge__(&self, py: Python<'_>, other: Ordered) -> PyResult<Self> {
         self.gte(py, other)
     }
 
-    fn __rge__(&self, py: Python<'_>, other: Numeric) -> PyResult<Self> {
+    fn __rge__(&self, py: Python<'_>, other: Ordered) -> PyResult<Self> {
         self.lte(py, other)
     }
 
@@ -524,7 +535,7 @@ impl LogicalExpr {
         })
     }
 
-    fn and(&self, py: Python<'_>, other: Boolish) -> PyResult<Self> {
+    fn and_(&self, py: Python<'_>, other: Boolish) -> PyResult<Self> {
         let expr: LogicalExpr = other.into();
 
         Ok(Self::Binary {
@@ -535,14 +546,14 @@ impl LogicalExpr {
     }
 
     fn __and__(&self, py: Python<'_>, other: Boolish) -> PyResult<Self> {
-        self.and(py, other)
+        self.and_(py, other)
     }
 
     fn __rand__(&self, py: Python<'_>, other: Boolish) -> PyResult<Self> {
-        self.and(py, other)
+        self.and_(py, other)
     }
 
-    fn or(&self, py: Python<'_>, other: Boolish) -> PyResult<Self> {
+    fn or_(&self, py: Python<'_>, other: Boolish) -> PyResult<Self> {
         let expr: LogicalExpr = other.into();
 
         Ok(Self::Binary {
@@ -553,11 +564,11 @@ impl LogicalExpr {
     }
 
     fn __or__(&self, py: Python<'_>, other: Boolish) -> PyResult<Self> {
-        self.or(py, other)
+        self.or_(py, other)
     }
 
     fn __ror__(&self, py: Python<'_>, other: Boolish) -> PyResult<Self> {
-        self.or(py, other)
+        self.or_(py, other)
     }
 
     fn starts_with(&self, py: Python<'_>, other: Stringy) -> PyResult<Self> {
@@ -623,13 +634,16 @@ impl LogicalExpr {
     /// Otherwise, the scoring expression is unchanged (multiplied by 1).
     fn boost(&self, py: Python<'_>, condition: FlexibleExpr, boost: Numeric) -> PyResult<Self> {
         let condition_expr = Into::<LogicalExpr>::into(condition);
-        let choose_expr =
-            condition_expr.choose(py, FlexibleExpr::Expr(boost.into()), FlexibleExpr::Int(1))?;
+        let choose_expr = condition_expr.choose(
+            py,
+            FlexibleExpr::Expr(boost.into()),
+            FlexibleExpr::Float(1.0),
+        )?;
         let choose_numeric = Numeric::Expr(choose_expr);
         self.mul(py, choose_numeric)
     }
 
-    fn min(&self, py: Python<'_>, other: Numeric) -> PyResult<Self> {
+    fn min(&self, py: Python<'_>, other: Ordered) -> PyResult<Self> {
         Ok(Self::Binary {
             left: Py::new(py, self.clone())?,
             op: BinaryOperator::Min,
@@ -637,7 +651,7 @@ impl LogicalExpr {
         })
     }
 
-    fn max(&self, py: Python<'_>, other: Numeric) -> PyResult<Self> {
+    fn max(&self, py: Python<'_>, other: Ordered) -> PyResult<Self> {
         Ok(Self::Binary {
             left: Py::new(py, self.clone())?,
             op: BinaryOperator::Max,
