@@ -41,8 +41,17 @@ pub fn topk_sdk(m: &Bound<'_, PyModule>) -> PyResult<()> {
     module!(m, "data", data::pymodule)?;
     module!(m, "error", error::pymodule)?;
 
-    // client
-    m.add_class::<client::Client>()?;
+    // clients
+    m.add_class::<client::sync::Client>()?;
+    m.add_class::<client::sync::CollectionsClient>()?;
+    m.add_class::<client::sync::CollectionClient>()?;
+
+    m.add_class::<client::r#async::AsyncClient>()?;
+    m.add_class::<client::r#async::AsyncCollectionsClient>()?;
+    m.add_class::<client::r#async::AsyncCollectionClient>()?;
+
+    // classes
+    m.add_class::<data::collection::Collection>()?;
 
     Ok(())
 }
