@@ -5,13 +5,13 @@ from topk_sdk.data import bytes
 def test_bytes_from_list():
     result = bytes([0, 1, 255, 128])
     assert result is not None
-    assert str(result) == str(b'\x00\x01\xff\x80')
+    assert str(result) == str(b"\x00\x01\xff\x80")
 
 
 def test_bytes_from_bytes():
     result = bytes(b"\x00\x01\xff\x80")
     assert result is not None
-    assert str(result) == str(b'\x00\x01\xff\x80')
+    assert str(result) == str(b"\x00\x01\xff\x80")
 
 
 def test_bytes_invalid_input():
