@@ -1,6 +1,15 @@
-import pytest
-from topk_sdk import error
-from topk_sdk.query import field, filter, fn, not_, select, literal, match, min, max, abs
+from topk_sdk.query import (
+    abs,
+    field,
+    filter,
+    fn,
+    literal,
+    match,
+    max,
+    min,
+    not_,
+    select,
+)
 
 from . import ProjectContext
 from .utils import dataset, doc_ids
@@ -10,9 +19,7 @@ def test_query_lte(ctx: ProjectContext):
     collection = dataset.books.setup(ctx)
 
     result = ctx.client.collection(collection.name).query(
-        filter(field("published_year") <= 1950).topk(
-            field("published_year"), 100, True
-        )
+        filter(field("published_year") <= 1950).topk(field("published_year"), 100, True)
     )
 
     assert doc_ids(result) == {"1984", "pride", "hobbit", "moby", "gatsby"}
@@ -29,24 +36,31 @@ def test_query_and(ctx: ProjectContext):
 
     assert doc_ids(result) == {"1984"}
 
+
 def test_query_is_null(ctx: ProjectContext):
     collection = dataset.books.setup(ctx)
 
     result = ctx.client.collection(collection.name).query(
-        filter(field("nullable_embedding").is_null()).topk(field("published_year"), 100, True)
+        filter(field("nullable_embedding").is_null()).topk(
+            field("published_year"), 100, True
+        )
     )
 
     assert doc_ids(result) == {"pride", "gatsby", "moby", "hobbit", "lotr", "alchemist"}
 
+
 def test_query_is_not_null(ctx: ProjectContext):
     collection = dataset.books.setup(ctx)
 
     result = ctx.client.collection(collection.name).query(
-        filter(field("nullable_embedding").is_not_null()).topk(field("published_year"), 100, True)
+        filter(field("nullable_embedding").is_not_null()).topk(
+            field("published_year"), 100, True
+        )
     )
 
     assert doc_ids(result) == {"mockingbird", "1984", "catcher", "harry"}
 
+
 def test_query_not(ctx: ProjectContext):
     collection = dataset.books.setup(ctx)
 
@@ -68,6 +82,7 @@ def test_query_not(ctx: ProjectContext):
         "pride",
     }
 
+
 def test_query_choose_literal(ctx: ProjectContext):
     collection = dataset.books.setup(ctx)
 
@@ -83,6 +98,7 @@ def test_query_choose_literal(ctx: ProjectContext):
 
     assert doc_ids(result) == {"pride", "gatsby"}
 
+
 def test_query_choose_literal_and_field(ctx: ProjectContext):
     collection = dataset.books.setup(ctx)
 
@@ -91,8 +107,7 @@ def test_query_choose_literal_and_field(ctx: ProjectContext):
             love_score=field("summary")
             .match_all("love")
             .choose(field("published_year"), literal(10))
-        )
-        .topk(field("love_score"), 2, False)
+        ).topk(field("love_score"), 2, False)
     )
 
     assert result == [
@@ -100,6 +115,7 @@ def test_query_choose_literal_and_field(ctx: ProjectContext):
         {"_id": "pride", "love_score": 1813},
     ]
 
+
 def test_query_choose_field(ctx: ProjectContext):
     collection = dataset.books.setup(ctx)
 
@@ -108,8 +124,7 @@ def test_query_choose_field(ctx: ProjectContext):
             love_score=field("summary")
             .match_all("love")
             .choose(field("published_year"), field("published_year") / 10)
-        )
-        .topk(field("love_score"), 3, False)
+        ).topk(field("love_score"), 3, False)
     )
 
     assert result == [
@@ -118,6 +133,7 @@ def test_query_choose_field(ctx: ProjectContext):
         {"_id": "harry", "love_score": 199},
     ]
 
+
 def test_query_coalesce_nullable(ctx: ProjectContext):
     collection = dataset.books.setup(ctx)
 
@@ -132,6 +148,7 @@ def test_query_coalesce_nullable(ctx: ProjectContext):
         {"_id": "pride", "importance": 1.0},
     ]
 
+
 def test_query_coalesce_missing(ctx: ProjectContext):
     collection = dataset.books.setup(ctx)
 
@@ -146,6 +163,7 @@ def test_query_coalesce_missing(ctx: ProjectContext):
         {"_id": "pride", "importance": 1.0},
     ]
 
+
 def test_query_coalesce_non_nullable(ctx: ProjectContext):
     collection = dataset.books.setup(ctx)
 
@@ -160,6 +178,7 @@ def test_query_coalesce_non_nullable(ctx: ProjectContext):
         {"_id": "pride", "coalesced_year": 1813},
     ]
 
+
 def test_query_abs(ctx: ProjectContext):
     collection = dataset.books.setup(ctx)
 
@@ -176,13 +195,21 @@ def test_query_abs(ctx: ProjectContext):
         {"_id": "mockingbird", "abs_year": 30},
     ]
 
+
 def test_query_topk_min_max(ctx: ProjectContext):
     collection = dataset.books.setup(ctx)
 
     result = ctx.client.collection(collection.name).query(
         select(bm25_score=fn.bm25_score())
         .select(clamped_bm25_score=max(min(field("bm25_score"), 2.0), 1.6))
-        .filter(match("millionaire love consequences dwarves", field="summary", weight=1.0, all=False))
+        .filter(
+            match(
+                "millionaire love consequences dwarves",
+                field="summary",
+                weight=1.0,
+                all=False,
+            )
+        )
         .topk(field("clamped_bm25_score"), 5, False)
     )
 
@@ -201,3 +228,30 @@ def test_query_topk_min_max(ctx: ProjectContext):
 
     assert result[3]["_id"] == "pride"
     assert result[3]["clamped_bm25_score"] == 1.6
+
+
+def test_query_gt_and_lte_string(ctx: ProjectContext):
+    collection = dataset.books.setup(ctx)
+
+    result = ctx.client.collection(collection.name).query(
+        filter(field("_id").gt("moby") & (field("_id").lte("pride"))).topk(
+            field("published_year"), 100, True
+        )
+    )
+
+    assert doc_ids(result) == {"mockingbird", "pride"}
+
+
+def test_query_min_string(ctx: ProjectContext):
+    collection = dataset.books.setup(ctx)
+
+    result = ctx.client.collection(collection.name).query(
+        select("title", min_string=field("title").min("Oz")).topk(
+            field("published_year"), 2, True
+        )
+    )
+
+    assert result == [
+        {"_id": "pride", "title": "Pride and Prejudice", "min_string": "Oz"},
+        {"_id": "moby", "title": "Moby-Dick", "min_string": "Moby-Dick"},
+    ]
