@@ -1,6 +1,10 @@
 use serde::{de::DeserializeOwned, Serialize};
+use std::time::Duration;
+use tonic::Status;
 use tracing::error;
 
+pub const MAX_VECTOR_DIMENSION: u32 = 16_384; // Double the size of `8192` which is the largest widely used vector dimension.
+
 #[derive(Debug, thiserror::Error)]
 pub enum Error {
     #[error("lsn timeout")]
@@ -39,6 +43,9 @@ pub enum Error {
     #[error("request too large: {0}")]
     RequestTooLarge(String),
 
+    #[error("internal error: {0}")]
+    Internal(String),
+
     #[error("unexpected error: {0}")]
     Unexpected(String),
 
@@ -73,12 +80,39 @@ impl Error {
             Error::TransportError(_) => false,
             Error::MalformedResponse(_) => false,
             Error::Unexpected(_) => false,
+            Error::Internal(_) => false,
+        }
+    }
+
+    /// Returns how long the client should keep retrying this error, if applicable.
+    /// Returns `Some(duration)` for errors that require retrying for a minimum time,
+    /// or `None` if no specific retry duration is required.
+    pub fn retry_duration(&self) -> Option<Duration> {
+        match self {
+            // When quering using `lsn=N`, the client should retry for at least 2 seconds,
+            // since the guarantee for "default" consistency mode is ~1 second.
+            Error::QueryLsnTimeout => Some(Duration::from_millis(2_000)),
+            _ => None,
         }
     }
 }
 
 impl From<Status> for Error {
     fn from(status: Status) -> Self {
+        let request_id = status
+            .metadata()
+            .get("x-request-id")
+            .and_then(|v| v.to_str().ok())
+            .map(|v| v.to_string());
+
+        let append_request_id = |message: &str| {
+            if let Some(request_id) = request_id {
+                format!("{message} (x-request-id: {request_id})")
+            } else {
+                message.to_string()
+            }
+        };
+
         match CustomError::try_from(status) {
             // Custom error
             Ok(error) => match error.code() {
@@ -100,7 +134,8 @@ impl From<Status> for Error {
                 },
                 tonic::Code::OutOfRange => Error::RequestTooLarge(e.message().into()),
                 tonic::Code::PermissionDenied => Error::PermissionDenied,
-                _ => Error::Unexpected(format!("unexpected error: {:?}", e)),
+                tonic::Code::Internal => Error::Internal(append_request_id(e.message())),
+                _ => Error::Unexpected(append_request_id(&format!("{:?}", e))),
             },
         }
     }
@@ -134,6 +169,9 @@ pub enum SchemaValidationError {
     #[error("vector field `{field}` cannot be have zero dimension")]
     VectorDimensionCannotBeZero { field: String },
 
+    #[error("vector field `{field}` cannot have dimension greater than {MAX_VECTOR_DIMENSION}")]
+    VectorDimensionTooLarge { field: String, dimension: u32 },
+
     #[error("Invalid semantic index for field `{field}. Error: {error}`")]
     InvalidSemanticIndex { field: String, error: String },
 }
@@ -159,6 +197,11 @@ pub enum DocumentValidationError {
         field: String,
     },
 
+    InvalidFieldName {
+        doc_id: String,
+        field: String,
+    },
+
     InvalidDataType {
         doc_id: String,
         field: String,
@@ -179,7 +222,19 @@ pub enum DocumentValidationError {
         reason: String,
     },
 
+    InvalidStructDepth {
+        doc_id: String,
+        field: String,
+        reason: String,
+    },
+
     NoDocuments,
+
+    DocumentTooLarge {
+        doc_id: String,
+        max_size_bytes: u64,
+        got_size_bytes: u64,
+    },
 }
 
 #[derive(PartialEq, Debug, serde::Serialize, serde::Deserialize)]
@@ -218,6 +273,10 @@ impl<T: Serialize + DeserializeOwned> ValidationErrorBag<T> {
     pub fn len(&self) -> usize {
         self.0.len()
     }
+
+    pub fn iter(&self) -> std::slice::Iter<'_, T> {
+        self.0.iter()
+    }
 }
 
 impl<T: Serialize> From<ValidationErrorBag<T>> for tonic::Status {
@@ -240,8 +299,6 @@ impl<T: Serialize + DeserializeOwned> TryFrom<tonic::Status> for ValidationError
     }
 }
 
-use tonic::Status;
-
 #[derive(Clone, PartialEq, Eq)]
 pub struct CustomError {
     /// Error message
