@@ -1,5 +1,10 @@
+use bytemuck::{cast_slice, cast_vec};
+use bytes::Bytes;
+use std::collections::HashMap;
+
 use crate::proto::data::v1::{
-    data_ext::IntoListValues, list, sparse_vector, value, vector, List, Null, SparseVector, Value,
+    data_ext::IntoListValues, list, sparse_vector, value, vector, List, Null, SparseVector, Struct,
+    Value,
 };
 
 impl Value {
@@ -9,6 +14,13 @@ impl Value {
         }
     }
 
+    pub fn as_null(&self) -> Option<()> {
+        match &self.value {
+            Some(value::Value::Null(_)) => Some(()),
+            _ => None,
+        }
+    }
+
     pub fn bool(value: bool) -> Self {
         Value {
             value: Some(value::Value::Bool(value)),
@@ -136,13 +148,13 @@ impl Value {
     }
 
     /// Alias for `binary`
-    pub fn bytes(value: Vec<u8>) -> Self {
+    pub fn bytes(value: impl Into<Bytes>) -> Self {
         Value::binary(value)
     }
 
-    pub fn binary(value: Vec<u8>) -> Self {
+    pub fn binary(value: impl Into<Bytes>) -> Self {
         Value {
-            value: Some(value::Value::Binary(value)),
+            value: Some(value::Value::Binary(value.into())),
         }
     }
 
@@ -153,6 +165,22 @@ impl Value {
         }
     }
 
+    /// Create a struct value from a map of values.
+    pub fn r#struct<K: Into<String>>(values: impl IntoIterator<Item = (K, Value)>) -> Self {
+        Value {
+            value: Some(value::Value::Struct(Struct {
+                fields: values.into_iter().map(|(k, v)| (k.into(), v)).collect(),
+            })),
+        }
+    }
+
+    pub fn as_struct(&self) -> Option<&HashMap<String, Value>> {
+        match &self.value {
+            Some(value::Value::Struct(s)) => Some(&s.fields),
+            _ => None,
+        }
+    }
+
     /// Create a list value from a vector of values.
     pub fn list<T: IntoListValues>(values: T) -> Self {
         Value {
@@ -172,6 +200,16 @@ impl Value {
         }
     }
 
+    pub fn as_i8_list(&self) -> Option<&[i8]> {
+        match &self.value {
+            Some(value::Value::List(list)) => match &list.values {
+                Some(list::Values::I8(v)) => Some(&v.as_ref()),
+                _ => None,
+            },
+            _ => None,
+        }
+    }
+
     pub fn as_f32_list(&self) -> Option<&[f32]> {
         match &self.value {
             Some(value::Value::List(list)) => match &list.values {
@@ -226,8 +264,11 @@ impl value::Value {
                 Some(list::Values::F32(_)) => "list<f32>".to_string(),
                 Some(list::Values::F64(_)) => "list<f64>".to_string(),
                 Some(list::Values::String(_)) => "list<string>".to_string(),
+                Some(list::Values::U8(_)) => "list<u8>".to_string(),
+                Some(list::Values::I8(_)) => "list<i8>".to_string(),
                 _ => "null_list".to_string(),
             },
+            value::Value::Struct(_) => "struct<string, Value>".to_string(),
             value::Value::Null(_) => "null".to_string(),
         }
     }
@@ -245,6 +286,12 @@ impl From<String> for Value {
     }
 }
 
+impl From<()> for Value {
+    fn from(_: ()) -> Self {
+        Value::null()
+    }
+}
+
 impl From<&str> for Value {
     fn from(value: &str) -> Self {
         Value::string(value.to_string())
@@ -287,6 +334,18 @@ impl From<f64> for Value {
     }
 }
 
+impl From<Vec<u32>> for Value {
+    fn from(value: Vec<u32>) -> Self {
+        Value::list(value)
+    }
+}
+
+impl From<Vec<u64>> for Value {
+    fn from(value: Vec<u64>) -> Self {
+        Value::list(value)
+    }
+}
+
 impl From<Vec<f32>> for Value {
     fn from(value: Vec<f32>) -> Self {
         Value::list(value)
@@ -299,6 +358,23 @@ impl From<Vec<u8>> for Value {
     }
 }
 
+impl From<Vec<i8>> for Value {
+    fn from(value: Vec<i8>) -> Self {
+        Value::list(value)
+    }
+}
+
+impl From<Vec<&str>> for Value {
+    fn from(value: Vec<&str>) -> Self {
+        Value::list(
+            value
+                .into_iter()
+                .map(|s| s.to_string())
+                .collect::<Vec<String>>(),
+        )
+    }
+}
+
 impl From<Vec<String>> for Value {
     fn from(value: Vec<String>) -> Self {
         Value::list(value)
@@ -313,6 +389,12 @@ impl From<SparseVector> for Value {
     }
 }
 
+impl From<HashMap<String, Value>> for Value {
+    fn from(value: HashMap<String, Value>) -> Self {
+        Value::r#struct(value)
+    }
+}
+
 impl<T: Into<Value>> From<Option<T>> for Value {
     fn from(value: Option<T>) -> Self {
         match value {
@@ -321,3 +403,27 @@ impl<T: Into<Value>> From<Option<T>> for Value {
         }
     }
 }
+
+impl From<list::I8> for Vec<i8> {
+    fn from(v: list::I8) -> Self {
+        cast_vec(v.values)
+    }
+}
+
+impl AsRef<[i8]> for list::I8 {
+    fn as_ref(&self) -> &[i8] {
+        cast_slice(&self.values)
+    }
+}
+
+impl Struct {
+    pub fn depth(&self) -> usize {
+        let mut depth = 1;
+        for (_, value) in &self.fields {
+            if let Some(value::Value::Struct(s)) = &value.value {
+                depth = s.depth() + 1;
+            }
+        }
+        depth
+    }
+}
