@@ -7,6 +7,17 @@ macro_rules! doc {
     };
 }
 
+#[macro_export]
+macro_rules! r#struct {
+    ($($field:expr => $value:expr),* $(,)?) => {
+        topk_rs::proto::v1::data::Value::r#struct(
+            std::collections::HashMap::from_iter(
+                [$(($field.to_string(), $value.into())),*]
+            )
+        )
+    };
+}
+
 #[macro_export]
 macro_rules! schema {
     () => {
